var test = require('tape'); // require('tap').test; // tape works in browserify, tap works in node 0.10
var shimmedKeys = require('./index.js');
var is = require('is-extended');
var keys = require('./shim.js');
var forEach = require('./shim-foreach.js');
var indexOf = require('indexof');

test('works', function (t) {
	var obj = {
		"str": "boz",
		"obj": {},
		"arr": [],
		"bool": true,
		"num": 42,
		"null": null,
		"undefined": undefined
	};
	var objKeys = ['str', 'obj', 'arr', 'bool', 'num', 'null', 'undefined'];

	t.test('exports a function', function (st) {
		if (Object.keys) {
			st.equal(Object.keys, shimmedKeys, 'Object.keys is supported and exported');
		} else {
			st.equal(keys, shimmedKeys, 'Object.keys is not supported; shim is exported');
		}
		st.end();
	});

	t.test('working with actual shim', function (st) {
		st.notEqual(Object.keys, keys, 'keys shim is not native Object.keys');
		st.end();
	});

	t.test('works with an object literal', function (st) {
		var theKeys = keys(obj);
		st.equal(is.isArray(theKeys), true, 'returns an array');
		st.deepEqual(theKeys, objKeys, 'Object has expected keys');
		st.end();
	});

	t.test('returns names which are own properties', function (st) {
		forEach(keys(obj), function (name) {
			st.equal(obj.hasOwnProperty(name), true);
		});
		st.end();
	});

	t.test('returns names which are enumerable', function (st) {
		var k, loopedValues = [];
		for (k in obj) {
			loopedValues.push(k);
		}
		forEach(keys(obj), function (name) {
			st.notEqual(indexOf(loopedValues, name), -1, name + ' is not enumerable');
		});
		st.end();
	});

	t.test('throws an error for a non-object', function (st) {
		st.throws(function () {
			return keys(42);
		}, RangeError, 'throws on a non-object');
		st.end();
	});
	t.end();
});

test('works with an object instance', function (t) {
	var Prototype = function () {};
	Prototype.prototype.foo = true;
	var obj = new Prototype();
	obj.bar = true;
	var theKeys = keys(obj);
	t.equal(is.isArray(theKeys), true, 'returns an array');
	t.deepEqual(theKeys, ['bar'], 'Instance has expected keys');
	t.end();
});

